@@ -1,4 +1,5 @@
 import json
+import datetime
 from types import TracebackType
 from typing import Optional, Type
 from urllib.parse import urlparse
@@ -129,6 +130,15 @@ class StreamChatAsync(StreamChatInterface):
         data = {"target_message_id": target_id, **options}
         return await self.post("moderation/unflag", data=data)
 
+    async def query_message_flags(self, filter_conditions, **options):
+        params = {
+            **options,
+            "filter_conditions": filter_conditions,
+        }
+        return await self.get(
+            "moderation/flags/message", params={"payload": json.dumps(params)}
+        )
+
     async def flag_user(self, target_id, **options):
         data = {"target_user_id": target_id, **options}
         return await self.post("moderation/flag", data=data)
@@ -299,8 +309,11 @@ class StreamChatAsync(StreamChatInterface):
 
         return await self.get("rate_limits", params)
 
-    async def search(self, filter_conditions, query, **options):
-        params = {**options, "filter_conditions": filter_conditions, "query": query}
+    async def search(self, filter_conditions, query, sort=None, **options):
+        if "offset" in options:
+            if sort or "next" in options:
+                raise ValueError("cannot use offset with sort or next parameters")
+        params = self.create_search_params(filter_conditions, query, sort, **options)
         return await self.get("search", params={"payload": json.dumps(params)})
 
     async def send_file(self, uri, url, name, user, content_type=None):
@@ -448,6 +461,40 @@ class StreamChatAsync(StreamChatInterface):
         """
         return await self.get("custom_role")
 
+    async def revoke_tokens(self, before):
+        """
+        Revokes tokens for an application
+        :param before: date before which the tokens are to be revoked, pass None to reset
+        """
+        if isinstance(before, datetime.datetime):
+            before = before.isoformat()
+
+        await self.update_app_settings({"revoke_tokens_issued_before": before})
+
+    async def revoke_user_token(self, user_id, before):
+        """
+        Revokes token for a user
+        :param user_id: user_id of user for which the token needs to be revoked
+        :param before: date before which the tokens are to be revoked, , pass None to reset
+        """
+        await self.revoke_users_token([user_id], before)
+
+    async def revoke_users_token(self, user_ids, before):
+        """
+        Revokes tokens for given users
+        :param user_ids: user_ids for user for whom the token needs to be revoked
+        :param before: date before which the tokens are to be revoked, pass None to reset
+        """
+        if isinstance(before, datetime.datetime):
+            before = before.isoformat()
+
+        updates = []
+        for user_id in user_ids:
+            updates.append(
+                {"id": user_id, "set": {"revoke_tokens_issued_before": before}}
+            )
+        await self.update_users_partial(updates)
+
     async def close(self):
         await self.session.close()
 
