@@ -38,12 +38,25 @@ class StreamChatInterface(abc.ABC):
                         sort_fields.append({"field": k, "direction": v})
         return sort_fields
 
-    def create_token(self, user_id, exp=None, **claims):
+    def create_token(self, user_id, exp=None, iat=None, **claims):
         payload = {**claims, "user_id": user_id}
         if exp is not None:
             payload["exp"] = exp
+        if iat is not None:
+            payload["iat"] = iat
         return jwt.encode(payload, self.api_secret, algorithm="HS256")
 
+    def create_search_params(self, filter_conditions, query, sort, **options):
+        params = options.copy()
+        if isinstance(query, str):
+            params.update({"query": query})
+        else:
+            params.update({"message_filter_conditions": query})
+        params.update(
+            {"filter_conditions": filter_conditions, "sort": self.normalize_sort(sort)}
+        )
+        return params
+
     def verify_webhook(self, request_body, x_signature):
         """
         Verify the signature added to a webhook event
@@ -113,6 +126,10 @@ class StreamChatInterface(abc.ABC):
     def unflag_message(self, target_id, **options):
         pass
 
+    @abc.abstractmethod
+    def query_message_flags(self, filter_conditions, **options):
+        pass
+
     @abc.abstractmethod
     def flag_user(self, target_id, **options):
         pass
@@ -413,3 +430,24 @@ class StreamChatInterface(abc.ABC):
         List custom roles of the app
         """
         pass
+
+    @abc.abstractmethod
+    def revoke_tokens(self, since):
+        """
+        Revoke tokens for an application issued since
+        """
+        pass
+
+    @abc.abstractmethod
+    def revoke_user_token(self, user_id, since):
+        """
+        Revoke tokens for a user issued since
+        """
+        pass
+
+    @abc.abstractmethod
+    def revoke_users_token(self, user_ids, since):
+        """
+        Revoke tokens for users issued since
+        """
+        pass
