@@ -3,6 +3,7 @@ from urllib.parse import urlparse
 from urllib.request import Request, urlopen
 
 import requests
+import datetime
 
 from stream_chat.__pkg__ import __version__
 from stream_chat.base.client import StreamChatInterface
@@ -124,6 +125,15 @@ class StreamChat(StreamChatInterface):
         data = {"target_message_id": target_id, **options}
         return self.post("moderation/unflag", data=data)
 
+    def query_message_flags(self, filter_conditions, **options):
+        params = {
+            **options,
+            "filter_conditions": filter_conditions,
+        }
+        return self.get(
+            "moderation/flags/message", params={"payload": json.dumps(params)}
+        )
+
     def flag_user(self, target_id, **options):
         data = {"target_user_id": target_id, **options}
         return self.post("moderation/flag", data=data)
@@ -294,8 +304,11 @@ class StreamChat(StreamChatInterface):
 
         return self.get("rate_limits", params)
 
-    def search(self, filter_conditions, query, **options):
-        params = {**options, "filter_conditions": filter_conditions, "query": query}
+    def search(self, filter_conditions, query, sort=None, **options):
+        if "offset" in options:
+            if sort or "next" in options:
+                raise ValueError("cannot use offset with sort or next parameters")
+        params = self.create_search_params(filter_conditions, query, sort, **options)
         return self.get("search", params={"payload": json.dumps(params)})
 
     def send_file(self, uri, url, name, user, content_type=None):
@@ -435,3 +448,37 @@ class StreamChat(StreamChatInterface):
         List custom roles of the app
         """
         return self.get("custom_role")
+
+    def revoke_tokens(self, before):
+        """
+        Revokes tokens for an application
+        :param before: date before which the tokens are to be revoked, to reset pass None
+        """
+        if isinstance(before, datetime.datetime):
+            before = before.isoformat()
+
+        self.update_app_settings({"revoke_tokens_issued_before": before})
+
+    def revoke_user_token(self, user_id, before):
+        """
+        Revokes token for a user
+        :param user_id: user_id of user for which the token needs to be revoked
+        :param before: date before which the tokens are to be revoked, to reset pass None
+        """
+        self.revoke_users_token([user_id], before)
+
+    def revoke_users_token(self, user_ids, before):
+        """
+        Revokes tokens for given users
+        :param user_ids: user_ids for user for whom the token needs to be revoked
+        :param before: date before which the tokens are to be revoked, to reset pass None
+        """
+        if isinstance(before, datetime.datetime):
+            before = before.isoformat()
+
+        updates = []
+        for user_id in user_ids:
+            updates.append(
+                {"id": user_id, "set": {"revoke_tokens_issued_before": before}}
+            )
+        self.update_users_partial(updates)
