@@ -1,14 +1,15 @@
 import sys
+from contextlib import suppress
 from operator import itemgetter
 
 import jwt
 import pytest
+import time
 import uuid
 from stream_chat.async_chat import StreamChatAsync
 from stream_chat.base.exceptions import StreamAPIException
 
 
-@pytest.mark.incremental
 class TestClient(object):
     def test_normalize_sort(self, client):
         expected = [
@@ -231,6 +232,22 @@ class TestClient(object):
         )
         await client.flag_message(msg_id, user_id=server_user["id"])
 
+    @pytest.mark.asyncio
+    async def test_query_message_flags(
+        self, event_loop, client, channel, random_user, server_user
+    ):
+        msg_id = str(uuid.uuid4())
+        await channel.send_message(
+            {"id": msg_id, "text": "helloworld"}, random_user["id"]
+        )
+        await client.flag_message(msg_id, user_id=server_user["id"])
+        response = await client.query_message_flags({"channel_cid": channel.cid})
+        assert len(response["flags"]) == 1
+        response = await client.query_message_flags(
+            {"user_id": {"$in": [random_user["id"]]}}
+        )
+        assert len(response["flags"]) == 1
+
     @pytest.mark.asyncio
     async def test_unflag_message(
         self, event_loop, client, channel, random_user, server_user
@@ -297,8 +314,36 @@ class TestClient(object):
             > response["server_side"]["GetRateLimits"]["remaining"]
         )
 
+    @pytest.mark.xfail
     @pytest.mark.asyncio
-    async def test_search(self, event_loop, client, channel, random_user):
+    async def test_search_with_sort(self, client, channel, random_user):
+        text = str(uuid.uuid4())
+        ids = ["0" + text, "1" + text]
+        await channel.send_message(
+            {"text": text, "id": ids[0]},
+            random_user["id"],
+        )
+        await channel.send_message(
+            {"text": text, "id": ids[1]},
+            random_user["id"],
+        )
+        response = await client.search(
+            {"type": "messaging"}, text, **{"limit": 1, "sort": [{"created_at": -1}]}
+        )
+        # searches all channels so make sure at least one is found
+        assert len(response["results"]) >= 1
+        assert response["next"] is not None
+        assert ids[1] == response["results"][0]["message"]["id"]
+        response = await client.search(
+            {"type": "messaging"}, text, **{"limit": 1, "next": response["next"]}
+        )
+        assert len(response["results"]) >= 1
+        assert response["previous"] is not None
+        assert response["next"] is None
+        assert ids[0] == response["results"][0]["message"]["id"]
+
+    @pytest.mark.asyncio
+    async def test_search(self, client, channel, random_user):
         query = "supercalifragilisticexpialidocious"
         await channel.send_message(
             {"text": f"How many syllables are there in {query}?"},
@@ -319,6 +364,45 @@ class TestClient(object):
         for message in response["results"]:
             assert query not in message["message"]["text"]
 
+    @pytest.mark.asyncio
+    async def test_search_message_filters(self, client, channel, random_user):
+        query = "supercalifragilisticexpialidocious"
+        await channel.send_message(
+            {"text": f"How many syllables are there in {query}?"},
+            random_user["id"],
+        )
+        await channel.send_message(
+            {"text": "Does 'cious' count as one or two?"}, random_user["id"]
+        )
+        response = await client.search(
+            {"type": "messaging"},
+            {"text": {"$q": query}},
+            **{
+                "limit": 2,
+                "offset": 0,
+            },
+        )
+        assert len(response["results"]) >= 1
+        assert query in response["results"][0]["message"]["text"]
+
+    @pytest.mark.asyncio
+    async def test_search_offset_with_sort(self, client):
+        query = "supercalifragilisticexpialidocious"
+        with pytest.raises(ValueError):
+            await client.search(
+                {"type": "messaging"},
+                query,
+                **{"limit": 2, "offset": 1, "sort": [{"created_at": -1}]},
+            )
+
+    @pytest.mark.asyncio
+    async def test_search_offset_with_next(self, client):
+        query = "supercalifragilisticexpialidocious"
+        with pytest.raises(ValueError):
+            await client.search(
+                {"type": "messaging"}, query, **{"limit": 2, "offset": 1, "next": query}
+            )
+
     @pytest.mark.asyncio
     async def test_query_channels_members_in(
         self, event_loop, client, fellowship_of_the_ring
@@ -364,14 +448,19 @@ class TestClient(object):
         assert "invalid SQS url" in response["error"]
 
     @pytest.mark.asyncio
+    @pytest.mark.skip(reason="slow and flaky due to waits")
     async def test_custom_permission_and_roles(self, client):
         name, role = "Something restricted", "god"
 
-        try:
-            client.delete_permission(name)
-            client.delete_role(role)
-        except:  # noqa
-            pass
+        def wait():
+            time.sleep(3)
+
+        with suppress(Exception):
+            await client.delete_permission(name)
+            wait()
+        with suppress(Exception):
+            await client.delete_role(role)
+            wait()
 
         custom = {
             "name": name,
@@ -381,8 +470,8 @@ class TestClient(object):
         }
 
         await client.create_permission(custom)
+        wait()
         response = await client.get_permission(name)
-        print(response)
         assert response["permission"]["name"] == name
         assert response["permission"]["custom"]
         assert not response["permission"]["owner"]
@@ -391,8 +480,8 @@ class TestClient(object):
         custom["owner"] = True
         await client.update_permission(name, custom)
 
+        wait()
         response = await client.get_permission(name)
-        print(response)
         assert response["permission"]["name"] == name
         assert response["permission"]["custom"]
         assert response["permission"]["owner"]
@@ -402,12 +491,15 @@ class TestClient(object):
         assert len(response["permissions"]) == 1
         assert response["permissions"][0]["name"] == name
         await client.delete_permission(name)
+        wait()
         response = await client.list_permissions()
         assert len(response["permissions"]) == 0
 
         await client.create_role(role)
+        wait()
         response = await client.list_roles()
         assert role in response["roles"]
         await client.delete_role(role)
+        wait()
         response = await client.list_roles()
         assert role not in response["roles"]
