@@ -1,15 +1,15 @@
+import sys
 from operator import itemgetter
-
-import uuid
+from contextlib import suppress
 
 import jwt
 import pytest
-import sys
+import time
+import uuid
 from stream_chat import StreamChat
 from stream_chat.base.exceptions import StreamAPIException
 
 
-@pytest.mark.incremental
 class TestClient(object):
     def test_normalize_sort(self, client):
         expected = [
@@ -192,6 +192,15 @@ class TestClient(object):
         channel.send_message({"id": msg_id, "text": "helloworld"}, random_user["id"])
         client.flag_message(msg_id, user_id=server_user["id"])
 
+    def test_query_message_flags(self, client, channel, random_user, server_user):
+        msg_id = str(uuid.uuid4())
+        channel.send_message({"id": msg_id, "text": "helloworld"}, random_user["id"])
+        client.flag_message(msg_id, user_id=server_user["id"])
+        response = client.query_message_flags({"channel_cid": channel.cid})
+        assert len(response["flags"]) == 1
+        response = client.query_message_flags({"user_id": {"$in": [random_user["id"]]}})
+        assert len(response["flags"]) == 1
+
     def test_unflag_message(self, client, channel, random_user, server_user):
         msg_id = str(uuid.uuid4())
         channel.send_message({"id": msg_id, "text": "helloworld"}, random_user["id"])
@@ -248,6 +257,33 @@ class TestClient(object):
             > response["server_side"]["GetRateLimits"]["remaining"]
         )
 
+    @pytest.mark.xfail
+    def test_search_with_sort(self, client, channel, random_user):
+        text = str(uuid.uuid4())
+        ids = ["0" + text, "1" + text]
+        channel.send_message(
+            {"text": text, "id": ids[0]},
+            random_user["id"],
+        )
+        channel.send_message(
+            {"text": text, "id": ids[1]},
+            random_user["id"],
+        )
+        response = client.search(
+            {"type": "messaging"}, text, **{"limit": 1, "sort": [{"created_at": -1}]}
+        )
+        # searches all channels so make sure at least one is found
+        assert len(response["results"]) >= 1
+        assert response["next"] is not None
+        assert ids[1] == response["results"][0]["message"]["id"]
+        response = client.search(
+            {"type": "messaging"}, text, **{"limit": 1, "next": response["next"]}
+        )
+        assert len(response["results"]) >= 1
+        assert response["previous"] is not None
+        assert response["next"] is None
+        assert ids[0] == response["results"][0]["message"]["id"]
+
     def test_search(self, client, channel, random_user):
         query = "supercalifragilisticexpialidocious"
         channel.send_message(
@@ -269,6 +305,42 @@ class TestClient(object):
         for message in response["results"]:
             assert query not in message["message"]["text"]
 
+    def test_search_message_filters(self, client, channel, random_user):
+        query = "supercalifragilisticexpialidocious"
+        channel.send_message(
+            {"text": f"How many syllables are there in {query}?"},
+            random_user["id"],
+        )
+        channel.send_message(
+            {"text": "Does 'cious' count as one or two?"}, random_user["id"]
+        )
+        response = client.search(
+            {"type": "messaging"},
+            {"text": {"$q": query}},
+            **{
+                "limit": 2,
+                "offset": 0,
+            },
+        )
+        assert len(response["results"]) >= 1
+        assert query in response["results"][0]["message"]["text"]
+
+    def test_search_offset_with_sort(self, client):
+        query = "supercalifragilisticexpialidocious"
+        with pytest.raises(ValueError):
+            client.search(
+                {"type": "messaging"},
+                query,
+                **{"limit": 2, "offset": 1, "sort": [{"created_at": -1}]},
+            )
+
+    def test_search_offset_with_next(self, client):
+        query = "supercalifragilisticexpialidocious"
+        with pytest.raises(ValueError):
+            client.search(
+                {"type": "messaging"}, query, **{"limit": 2, "offset": 1, "next": query}
+            )
+
     def test_query_channels_members_in(self, client, fellowship_of_the_ring):
         response = client.query_channels({"members": {"$in": ["gimli"]}}, {"id": 1})
         assert len(response["channels"]) == 1
@@ -302,14 +374,19 @@ class TestClient(object):
         assert response["status"] == "error"
         assert "invalid SQS url" in response["error"]
 
+    @pytest.mark.skip(reason="slow and flaky due to waits")
     def test_custom_permission_and_roles(self, client):
         name, role = "Something restricted", "god"
 
-        try:
+        def wait():
+            time.sleep(3)
+
+        with suppress(Exception):
             client.delete_permission(name)
+            wait()
+        with suppress(Exception):
             client.delete_role(role)
-        except:  # noqa
-            pass
+            wait()
 
         custom = {
             "name": name,
@@ -319,8 +396,8 @@ class TestClient(object):
         }
 
         client.create_permission(custom)
+        wait()
         response = client.get_permission(name)
-        print(response)
         assert response["permission"]["name"] == name
         assert response["permission"]["custom"]
         assert not response["permission"]["owner"]
@@ -329,8 +406,8 @@ class TestClient(object):
         custom["owner"] = True
         client.update_permission(name, custom)
 
+        wait()
         response = client.get_permission(name)
-        print(response)
         assert response["permission"]["name"] == name
         assert response["permission"]["custom"]
         assert response["permission"]["owner"]
@@ -340,12 +417,15 @@ class TestClient(object):
         assert len(response["permissions"]) == 1
         assert response["permissions"][0]["name"] == name
         client.delete_permission(name)
+        wait()
         response = client.list_permissions()
         assert len(response["permissions"]) == 0
 
         client.create_role(role)
+        wait()
         response = client.list_roles()
         assert role in response["roles"]
         client.delete_role(role)
+        wait()
         response = client.list_roles()
         assert role not in response["roles"]
